//Algorithmen.java
//Komponente zur Auswahl der Algorithmen

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;

class Algorithmen extends JPanel{
	
	private JComboBox 	c_kanten,			// Auswahl der Kantendetektoren 
					c_sigma,			// Auswahl der Standardabw. für Canny
					c_high,			// Auswahl des oberen Thresholds für Canny
					c_low,			// Auswahl des unteren Thresholds für Canny
					c_textur, 		// Auswahl der Texturalgorithmen
					c_law, 			// Auswahl der Matrizen für Laws Texture Measure
					c_direction, 		// Auswahl der Richtung der GLCM
					c_window,			// Auswahl der Fenstergröße
					c_distance;		// Auswahl des Abstands für die GLCM
						
	private JSlider 	s_kanten, 		// Wahl des Thresholds für die Kanten
					s_textur;			// Wahl des Thresholds für die Texturen
	
	private JLabel 	l_sliderValue,		// Label für die Bezeichnung der einzelnen
					l_canny,			// Komponenten
					l_sigma, 
					l_high, 
					l_low, 
					l_energy, 
					l_dir,
					l_text,
					l_window,
					l_distance;
				   		
	private float 		sigma = 0.4f;		// Standardabweichung für Canny	
	
	private int 		threshold = 128,	// Threshold für Kantenalgorithmen
					threshtext = 128,	// Threshold für Texturalgorithmen
					high = 35,		// oberer Threshold für Canny
					low = 20, 		// unterer Threshold für Canny
					energy = 1, 		// welche Matrix für Laws Texture Measure
					dir = 0,			// Richtung der GLCM
					window = 13,		// Fenstergröße
					algo = 0,			// welches statistische Maß für GLCM
					distance = 2;
						
	private BildKomp smoothed, kanten, textur, hintergrund;
	
	Algorithmen(Oberflaeche ob){
		
		Color farbeK = new Color(153, 181, 220);
		Color farbeT = new Color(173, 216, 230);
		
		this.smoothed = ob.getSmoothed();
		this.kanten = ob.getKanten();
		this.textur = ob.getTextur();
		this.hintergrund = ob.getHintergrund();
		
		//JComboBox
		//1: die Kantenalgorithmen
		c_kanten = new JComboBox();
		c_kanten.setMaximumRowCount(4);
		JScrollPane sp = new JScrollPane(c_kanten);
		c_kanten.addItem("Roberts Edge Detector");
		c_kanten.addItem("First Derivative");
		c_kanten.addItem("Second Derivative");
		c_kanten.addItem("Prewitt Edge Detector");
		c_kanten.addItem("Sobel Edge Detector");
		c_kanten.addItem("Robinson Edge Detector");
		c_kanten.addItem("Kirsch Edge Detector");
		c_kanten.addItem("Canny Edge Detector");
		c_kanten.addActionListener(new Kanten_ActionListener());
		
		//für die Parameter des Canny-Algorithmus
		//2: für die Standardabweichung
		c_sigma = new JComboBox();
		c_sigma.setEditable(true);
		c_sigma.setMaximumRowCount(3);
		sp = new JScrollPane(c_sigma);
		c_sigma.addItem(String.valueOf(0.4));
		c_sigma.addItem(String.valueOf(0.8));
		c_sigma.addItem(String.valueOf(1.0));
		c_sigma.addItem(String.valueOf(2.0));
		c_sigma.addItem(String.valueOf(4.0));
		c_sigma.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				String selected = (String)c_sigma.getSelectedItem();
				try{
     				sigma = Float.parseFloat(selected);
     				if(sigma <= 0.0f){
     					String message = "Standard deviation has to be > 0.";
     					if(sigma == 0.0f)
     						message = "Division by zero!\n"+message;
     					else
     						message = "Invalid input!\n"+message;
     					JOptionPane.showMessageDialog(null, message, "Error", JOptionPane.ERROR_MESSAGE);
     					sigma = 0.4f;
     					c_sigma.setSelectedIndex(0);
     				}
     				else
     					neuberechnenKanten();
     			} catch(NumberFormatException nfe){
     				String message = "'"+selected+"'"+ 
     								 " is not a valid input!\n"+
     								 "Input has to be a decimal.";
     				JOptionPane.showMessageDialog(null, message, "Error", JOptionPane.ERROR_MESSAGE);
     				sigma = 0.4f;
     				c_sigma.setSelectedIndex(0);
     			}
     		}
     	});
     	c_sigma.setEnabled(false);
     	
     	//3: unterer Threshold
		c_low = new JComboBox();
		c_low.setEditable(true);
		c_low.setMaximumRowCount(3);
		sp = new JScrollPane(c_low);
		c_low.addItem(String.valueOf(0));
		c_low.addItem(String.valueOf(20));
		c_low.addItem(String.valueOf(128));
		c_low.addItem(String.valueOf(255));
		c_low.setSelectedIndex(1);
		c_low.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				String selected = (String)c_low.getSelectedItem();
				try{
     				low = Integer.parseInt(selected);
     				
     				if((low < 0)|| (low > 255)){
     					String message = "Input range for the low threshold:\n"+
     								 "[0...255]";
     					JOptionPane.showMessageDialog(null, message, "Error", JOptionPane.ERROR_MESSAGE);
     					low=20;
     					c_low.setSelectedIndex(1);
     				}
     				else
     					neuberechnenKanten();
     			} catch(NumberFormatException nfe){
     				String message = "'"+selected+"'"+ 
     								 " is not a valid input!\n"+
     								 "Input has to be an integer.";
     				JOptionPane.showMessageDialog(null, message, "Error", JOptionPane.ERROR_MESSAGE);
     				low = 20;
     				c_low.setSelectedIndex(1);
     			}
     		}
     	});
     	c_low.setEnabled(false);
     	
     	//4: oberer Threshold
		c_high = new JComboBox();
		c_high.setEditable(true);
		c_high.setMaximumRowCount(3);
		sp = new JScrollPane(c_high);
		c_high.addItem(String.valueOf(0));
		c_high.addItem(String.valueOf(35));
		c_high.addItem(String.valueOf(128));
		c_high.addItem(String.valueOf(255));
		c_high.setSelectedIndex(1);
		c_high.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				String selected = (String)c_high.getSelectedItem();
				try{
     				high = Integer.parseInt(selected);
     				if((high < 0)|| (high > 255)){
     					String message = "Input range for the high threshold:\n"+
     								 	 "[0...255]";
     					JOptionPane.showMessageDialog(null, message, "Error", JOptionPane.ERROR_MESSAGE);
     					high=35;
     					c_high.setSelectedIndex(1);
     				}
     				else
     					neuberechnenKanten();
     				} catch(NumberFormatException nfe){
     					String message = "'"+selected+"'"+ 
     									 " is not a valid input!\n"+
     									 "Input has to be an integer.";
     					JOptionPane.showMessageDialog(null, message, "Error", JOptionPane.ERROR_MESSAGE);
     					high=35;
     					c_high.setSelectedIndex(1);
     				}   		
     		}
     	});
		c_high.setEnabled(false);
		
		//5: für die Texturalgorithmen
		c_textur = new JComboBox();
		c_textur.setMaximumRowCount(4);
		sp = new JScrollPane(c_textur);
		c_textur.addItem("Fractal Dimension");
		c_textur.addItem("GLCM Mean");
		c_textur.addItem("GLCM Standard Deviation");
		c_textur.addItem("GLCM Maximum Probability");
		c_textur.addItem("GLCM Energy");
		c_textur.addItem("GLCM Contrast");
		c_textur.addItem("GLCM Homogeneity");
		c_textur.addItem("GLCM Entropy");
		c_textur.addItem("Law's Texture Energy Measures");
		c_textur.addActionListener(new Textur_ActionListener());
		
		//6: für Laws Texturbeschreibung
		//Wahl der Filtermasken
		c_law = new JComboBox();
		c_law.setMaximumRowCount(4);
		sp = new JScrollPane(c_law);
		c_law.addItem("E5E5");
		c_law.addItem("E5L5");
		c_law.addItem("E5R5");
		//c_law.addItem("L5L5");
		c_law.addItem("L5E5");
		c_law.addItem("L5R5");
		c_law.addItem("R5R5");
		c_law.addItem("R5E5");
		c_law.addItem("R5L5");
		c_law.setSelectedIndex(energy);
		c_law.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				energy = c_law.getSelectedIndex();
     			neuberechnenTextur();
     		}
     	});
		c_law.setEnabled(false);
		
		//7: für die Richtung der Berechnung der GLCMs
		c_direction = new JComboBox();
		c_direction.setMaximumRowCount(4);
		sp = new JScrollPane(c_direction);
		c_direction.addItem("  0°");
		c_direction.addItem(" 45°");
		c_direction.addItem(" 90°");
		c_direction.addItem("135°");
		c_direction.setSelectedIndex(dir);
		c_direction.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				//0 entspricht 0°
				//1 entspricht 45°
				//2 entspricht 90°
				//3 entspricht 135°
				dir = c_direction.getSelectedIndex();
     			neuberechnenTextur();
     		}
     	});
		c_direction.setEnabled(false);
		
		//8: Größe des Fensters
		c_window = new JComboBox();
		c_window.setMaximumRowCount(4);
		sp = new JScrollPane(c_window);
		c_window.addItem("  7x7");
		c_window.addItem("  9x9");
		c_window.addItem("13x13");
		c_window.addItem("21x21");
		c_window.setSelectedIndex(0);
		c_window.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				int i=c_window.getSelectedIndex();
				switch(i){
					case 0: window=7; break;
					case 1: window=9; break;
					case 2: window=13; break;
					case 3: window=21; break;
					default: window=13;
				}
				neuberechnenTextur();
     		}
     	});
       	c_window.setEnabled(false);
       	
       	//7: für die Richtung der Berechnung der GLCMs
		c_distance = new JComboBox();
		c_distance.setMaximumRowCount(4);
		sp = new JScrollPane(c_direction);
		c_distance.addItem(String.valueOf(1));
		c_distance.addItem(String.valueOf(2));
		c_distance.addItem(String.valueOf(4));
		c_distance.addItem(String.valueOf(6));
		c_distance.setSelectedIndex(1);
		c_distance.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				String selected = (String)c_distance.getSelectedItem();
     			distance = Integer.parseInt(selected);
     			neuberechnenTextur();
     		}
     	});
		c_distance.setEnabled(false);
		
		//JSlider
		//1: Threshold für die Kanten
		s_kanten = new JSlider(JSlider.HORIZONTAL, 0, 255, threshold);
		s_kanten.putClientProperty( "JSlider.isFilled", Boolean.TRUE );
		s_kanten.setValueIsAdjusting(true);
		s_kanten.setPaintTicks(true);
		s_kanten.setPaintLabels(true);
		s_kanten.setMajorTickSpacing(255);
		s_kanten.setMinorTickSpacing(15);
		s_kanten.addChangeListener(new Slider_ChangeListener());
		
		//2: Threshold für die Texturen
		s_textur = new JSlider(JSlider.HORIZONTAL, 0, 255, threshtext);
		s_textur.putClientProperty( "JSlider.isFilled", Boolean.TRUE );
		s_textur.setValueIsAdjusting(true);
		s_textur.setPaintTicks(true);
		s_textur.setPaintLabels(true);
		s_textur.setMajorTickSpacing(255);
		s_textur.setMinorTickSpacing(15);
		s_textur.addChangeListener(new Slider_ChangeListener());
		
		//JLabel
		//1: für die Kantenalgorithmen
		JLabel l_kanten = new JLabel("Choose an edge detector");
		
		//2: für die Anzeige der Sliderwerte
		l_sliderValue = new JLabel("Threshold value: "+s_kanten.getValue());
		
		//3: für den Canny-Algorithmus
		l_canny = new JLabel("Parameters for Canny Edge Detector");
		l_canny.setEnabled(false);
		
		//4: Standardabweichung für Canny
		l_sigma = new JLabel("Standard deviation (float > 0)");
		l_sigma.setEnabled(false);
		
		//5: unterer Threshold
		l_low = new JLabel("Low threshold (integer \u2208 [0, ..., 255])");
		l_low.setEnabled(false);
		
		//6: oberer Threshold
		l_high = new JLabel("High threshold (integer \u2208 [0, ..., 255])");
		l_high.setEnabled(false);
		
		//7: Texturalgorithmen
		JLabel l_textur = new JLabel("Choose a textur algorithm");
		
		//8: Laws Texturbeschreibung
		l_energy = new JLabel("Filter mask:");
		l_energy.setEnabled(false);
		
		//9: Richtung für GLCM
		l_dir = new JLabel("Direction:");
		l_dir.setEnabled(false);
		
		//10: für die Anzeige der Sliderwerte
		l_text = new JLabel("Threshold value: "+s_textur.getValue());
		
		//11: für die Fenstergröße
		l_window = new JLabel("Window size: ");
		l_window.setEnabled(false);
		
		//12: für die Abstände in den GLCM
		l_distance = new JLabel("Distance: ");
		l_distance.setEnabled(false);
				
		JPanel p = new JPanel();
		p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
		//p.add(Box.createVerticalStrut(12));
		
		JPanel l = new JPanel();
		l.setBackground(farbeK);
		l.setLayout(new GridLayout(0,1));
		l.add(l_kanten);
		l.add(c_kanten);
				
		JPanel t = new JPanel();
		t.setBackground(farbeK);
		t.setLayout(new FlowLayout(FlowLayout.LEFT));
		t.add(l);
		p.add(t);
		
		t = new JPanel();
		t.setBackground(farbeK);
		t.setLayout(new FlowLayout(FlowLayout.LEFT));
		t.add(l_sliderValue);
		p.add(t);
		
		s_kanten.setBackground(farbeK);
		p.add(s_kanten);
		
		l = new JPanel();
		l.setBackground(farbeK);
		l.setLayout(new GridLayout(0,1));
		l.add(l_canny);
		l.add(l_sigma);
		l.add(c_sigma);
		l.add(l_low);
		l.add(c_low);
		l.add(l_high);
		l.add(c_high);
				
		t = new JPanel();
		t.setBackground(farbeK);
		t.setLayout(new FlowLayout(FlowLayout.LEFT));
		t.add(l);
		p.add(t);
		p.add(Box.createVerticalStrut(6));
		
/*---------------------------------------------------------*/
		
		l = new JPanel();
		l.setLayout(new GridLayout(0,1));
		l.setBackground(farbeT);
		l.add(l_textur);
		l.add(c_textur);
		
		t = new JPanel();
		t.setBackground(farbeT);
		t.setLayout(new FlowLayout(FlowLayout.LEFT));
		t.add(l);
		p.add(t);
		
		l = new JPanel();
		l.setBackground(farbeT);
		l.setLayout(new GridLayout(0,2, 50, 0));
		l.add(l_dir);
		l.add(c_direction);
		l.add(l_distance);
		l.add(c_distance);
		l.add(l_window);
		l.add(c_window);
		l.add(l_energy);
		l.add(c_law);
		
		t = new JPanel();
		t.setBackground(farbeT);
		t.setLayout(new FlowLayout(FlowLayout.LEFT));
		t.add(l);
		p.add(t);
		
		t = new JPanel();
		t.setBackground(farbeT);
		t.setLayout(new FlowLayout(FlowLayout.LEFT));
		t.add(l_text);
		p.add(t);
		
		s_textur.setBackground(farbeT);
		p.add(s_textur);
		
		add(p, BorderLayout.NORTH);
		
		neuberechnen();
	}
	
	private void activateCanny(boolean activ){
		c_sigma.setEnabled(activ);
		c_low.setEnabled(activ);
		c_high.setEnabled(activ);
		l_canny.setEnabled(activ);
		l_sigma.setEnabled(activ);
		l_low.setEnabled(activ);
		l_high.setEnabled(activ);
		//für Rest disablen
		l_sliderValue.setEnabled(!activ);
		s_kanten.setEnabled(!activ);
	}
	
	private void activateLaw(boolean activ){
		l_energy.setEnabled(activ);
		c_law.setEnabled(activ);
	}
	
	private void activateGLCM(boolean activ){
		l_dir.setEnabled(activ);
		c_direction.setEnabled(activ);
		l_distance.setEnabled(activ);
		c_distance.setEnabled(activ);
	}
	
	private void activateWindow(boolean activ){
		c_window.setEnabled(activ);
		l_window.setEnabled(activ);
	}
	
	private void neuberechnenKanten(){
		SwingUtilities.invokeLater(new Runnable(){
			Image img = smoothed.getImage();
			public void run(){
				String str = (String)c_kanten.getSelectedItem();
     			if(str.equalsIgnoreCase("Canny Edge Detector"))
					//für Canny enablen
					activateCanny(true);
				else
					//alles für Canny disablen
     				activateCanny(false);
     					
				//Kantenberechnung
				Thread t = null;
				int select = c_kanten.getSelectedIndex();
				if(select != -1)
					t = kantenThread(select, img);
				if(t != null)
					t.start();
				try{
					t.join();
				}catch (InterruptedException e){}
				
				//Hintergrund
				neuberechnenHintergrund();
			}
		});
	}
	
	private void neuberechnenTextur(){
		SwingUtilities.invokeLater(new Runnable(){
			Image img = smoothed.getImage();
			public void run(){
					
				//Texturberechnung
				Thread t = null;
				int select = c_textur.getSelectedIndex();
				if(select != -1){
					t = texturThread(select, img);
					//t.setPriority(Thread.MIN_PRIORITY);
				}
				if(t != null)
					t.start();
				if(select == 0)
					activateWindow(false);
				else
					activateWindow(true);
				if(select < 8 && select > 0)
					activateGLCM(true);
				else
					activateGLCM(false);
				if(select == 8)
					activateLaw(true);
				else
					activateLaw(false);	
				try{
					t.join();
				}catch (InterruptedException e){}
				
				//Hintergrund
				neuberechnenHintergrund();
			}
		});
	}
	
	private void neuberechnenHintergrund(){
		Image img = smoothed.getImage();
		Image imgK = kanten.getImage();
		Image imgT = textur.getImage();
		SwingUtilities.invokeLater(new Hintergrund(img, imgK, imgT, hintergrund));
	}
	
	public void neuberechnen(){
		SwingUtilities.invokeLater(new Runnable(){
			Image img = smoothed.getImage();
			public void run(){
				
				//Kantenberechnung
				Thread t = null;
				int select = c_kanten.getSelectedIndex();
				if(select != -1)
					t = kantenThread(select, img);
				if(t != null)
					t.start();
				
				//Texturberechnung
				Thread h = null;
				select = c_textur.getSelectedIndex();
				if(select != -1)
					h = texturThread(select, img);
				if(h != null)
					h.start();
				//Auf Beenden von Kanten- und Texturberechnungsthreads warten
				try{
					t.join();
					h.join();
				}catch (InterruptedException e){}
				
				//Hintergrundberechnung
				neuberechnenHintergrund();
				
			}
		});
		
	}
	
	private Thread texturThread(int select, Image img){
		//GLCM 2.Element in ComboBox, einzelnen GLCM-Algos
		//von 0 bis 6
		algo = select - 1;
		switch(select){
			case 0: return new Thread(new Fractal(img, textur, threshtext));
			case 1: 
			case 2: 
			case 3: 
			case 4: 
			case 5: 
			case 6:	
			case 7: return new Thread(new GLCM(img, textur, algo, window, distance, dir, threshtext));
			case 8: return new Thread(new Energy(img, textur, window, energy, threshtext));
			default: return null;
		}
	}
	
	private Thread kantenThread(int select, Image img){
		switch (select){
			case 0: return new Thread(new Robert(img, kanten, threshold));
			case 1: return new Thread(new Gradient(img, kanten, threshold));
			case 2: return new Thread(new Gradient2(img, kanten, threshold));
			case 3: return new Thread(new Prewitt(img, kanten, threshold));
			case 4: return new Thread(new Sobel(img, kanten, threshold));
			case 5: return new Thread(new Robinson(img, kanten, threshold));
			case 6: return new Thread(new Kirsch(img, kanten, threshold));
			case 7: return new Thread(new Canny(img, kanten, sigma, low, high));
			default: return null;
		}
	}
			
     class Kanten_ActionListener implements ActionListener{
     	public void actionPerformed (ActionEvent ie){
     		neuberechnenKanten();
		}	
	}
	
	class Textur_ActionListener implements ActionListener{
     	public void actionPerformed (ActionEvent ie){
     		neuberechnenTextur();
		}
	}
  
     class Slider_ChangeListener implements ChangeListener{
     	public void stateChanged(ChangeEvent ce){
     		JSlider s = (JSlider)ce.getSource();
     		if(s==s_kanten){
     			if(s.getValueIsAdjusting() == false){
     				threshold = s.getValue();
     				l_sliderValue.setText("Threshold value: "+threshold);
     				neuberechnenKanten();
     			}
			}
			if(s==s_textur){
				if(s.getValueIsAdjusting() == false){
     				threshtext = s.getValue();
     				l_text.setText("Threshold value: "+threshtext);
     				neuberechnenTextur();
				}
			}
		}
     }
}