//Canny.java
//Canny Edge Detector
import java.awt.image.*;
import java.awt.Image;
import javax.swing.*;
import java.util.*;

class Canny extends Algorithmus{
	
	private int low, high;	//oberer, unterer Schwellwert
	private float sigma; //Standardabweichung für die Gaußfkt.
	
	//größte mögliche Filtermaske
	private static final int max_maske = 20;
	
	//Skalieren der floating-point magnituden auf 8 bits
	private static final float mag_scale = 20.0f; 
	
	private static int width = 0;
	
	private BufferedImage buffImgHelp;
	private WritableRaster wraster;
	private int breite_maske;
	
	Canny(Image img, BildKomp ziel, float sigma, int low, int high){
		super(img, ziel);
		this.sigma = sigma;
		this.low = low;
		this.high = high;
	}
		
	protected Image berechne(){
		
		float[] gau = new float[max_maske];		
		float[] dgau = new float[max_maske];
		filterBerechnen(gau, dgau);
		
		BufferedImage buffImgHelp = Rand.kopiereRand(img, width*2, BufferedImage.TYPE_BYTE_GRAY);
		
		//Daten aus Bild holen
		Raster raster = buffImgHelp.getData();
		WritableRaster wraster = raster.createCompatibleWritableRaster();
		
		alt_breite = breite;
		alt_hoehe = hoehe;
		
		breite = buffImgHelp.getWidth(null);
		hoehe = buffImgHelp.getHeight(null);
		
		byte[] original = (byte[])raster.getDataElements(0, 0, breite, hoehe, null);
		original = berechneCanny(original, gau, dgau);
			
		//neu berechnete Bild 			
		wraster.setDataElements(0,0, breite, hoehe, original);
		buffImgHelp.setData(wraster);
		
		return buffImgHelp.getSubimage(width, width, alt_breite, alt_hoehe);	
	}
	
	private byte[] berechneCanny(byte[] original, float[] gau, float[] dgau){
		
		//local image space createn
		int[][] magim = new int[breite][hoehe];
						
		//Canny-Filter anwenden
		canny(original, magim, gau, dgau);
		
		//Hysteresis thresholding der Kantenpixel 
		int[][] im = hysteresis (magim);

		return Util.matrixZuFeld(im);
	}
	
	private void filterBerechnen(float[] gau, float[] dgau){
		breite_maske=0;
				
		//Filtermasken für die Gauß-Funktion und
		//die 1. Ableitung der Gauß-Funktion anlegen
		
		for(int i=0; i < max_maske; i++){
			gau[i] = meanGauss(i);
			if(gau[i] < 0.005){
				breite_maske = i;
				break;
			}
			dgau[i] = dGauss(i);
		}
		
		width = breite_maske/2;
	}
	
	private void canny(byte[] original, int[][] magim, float[] gau, float[] dgau){
	
		//Faltung des Originalbildes mit der Gaußmaske in
		//x- und y-Richtung
		float[][] scx = new float[breite][hoehe];
		float[][] scy = new float[breite][hoehe];
		separableConvolution(original, gau, breite_maske, scx, scy);
		
		
		//Faltung der geglätteten Daten mit der 1. Ableitung
		float[][] dx = dxy_seperableConvolution(scx, dgau, breite_maske, true);
		float[][] dy = dxy_seperableConvolution(scy, dgau, breite_maske, false);
				
		//Non-maximum suppression - Kantenpixel sollten lokale
		//Maxima sein
		nonmaxSuppress(dx, dy, magim);
	}
	
	
	//Gauß-Funktion
	private float gauss(float x){
		
		x = (float)Math.exp((double) ((-x*x)/(2*sigma*sigma)));
    	
    	return x;
    }
	
	private float meanGauss(float x){
		
		x = (gauss(x) + gauss(x+0.5f) + gauss(x-0.5f))/3.0f;
		x /= Math.PI * 2.0f*sigma*sigma;
		
		return x;
	}
	
	//1. Ableitung der Gauß-Funktion
	private float dGauss (float x){
	
		return -x/(sigma*sigma) * gauss(x);
	}
	
	private void separableConvolution(byte[] original, float[] gau, int breite_maske, float[][] scx, float[][] scy){
		
		int[][] hilfe = Util.feldZuIntMatrix(original, breite, hoehe);
		
		for(int i=breite-1; i >= 0; i--)
			for(int j=hoehe-1; j >= 0; j--){
				float x = gau[0]*hilfe[i][j];
				float y = gau[0]*hilfe[i][j];
				for(int k=1; k < breite_maske; k++){
					int I1 = (j+k) % hoehe;
					int I2 = (j-k+hoehe) % hoehe;
					y += gau[k]*hilfe[i][I1] + gau[k]*hilfe[i][I2];
	      			I1 = (i+k) % breite; 
	      			I2 = (i-k+breite) % breite;
	      			x += gau[k]*hilfe[I1][j] + gau[k]*hilfe[I2][j];
	      		}
	      		scx[i][j] = x;
	      		scy[i][j] = y;
	      	}
	 }
	 
	 private float[][] dxy_seperableConvolution(float[][] sc, float[] dgau, int breite_maske, boolean richtung){
	 	
	 	float[][] abl = new float[breite][hoehe];
	 	
	 	for(int i=breite-1; i >=0 ; i--)
	 		for(int j=hoehe-1; j  >= 0; j--){
	 			float x = 0f;
	 			for (int k=1; k < breite_maske; k++){
	      			if (richtung){
						int I1 = (j+k) % hoehe;
						int I2 = (j-k+hoehe) % hoehe;
						x += -dgau[k]*sc[i][I1] + dgau[k]*sc[i][I2];
	      			}
	     			else{
						int I1 = (i+k) % breite; 
						int I2 = (i-k+breite) % breite;
						x += -dgau[k]*sc[I1][j] + dgau[k]*sc[I2][j];
	      			}
	    		}
	    		abl[i][j] = x;
	  		}
	  		
	  	return abl;
	}
	      	
	
	private float norm(float x, float y){
		
		return (float) Math.sqrt(x*x + y*y);
	}
	
	private void nonmaxSuppress(float[][] dx, float[][] dy, int[][] mag){
	
		float xx, yy, g1, g2, g3, g4, g;
		
			
	for(int i=mag.length-2; i > 0; i--){
	  for (int j=mag[0].length-2; j > 0 ; j--){
	    	mag[i][j] = 0;

			float xc = dx[i][j];
	    	float yc = dy[i][j];
	    	if(Math.abs(xc)<0.01 && Math.abs(yc)<0.01) continue;

	    	g  = norm (xc, yc);

			//folge der Richtung des Gradienten 
			//wie durch die Richtung des Vectors (xc, yc) angedeutet, 
			//behalte Pixel die lokale Maxima sind

	    	if(Math.abs(yc) > Math.abs(xc)){

				//die y-Komponente ist größer, 
				//also ist der Gradient senkrecht ausgerichtet
	      		xx = Math.abs(xc)/Math.abs(yc);
	      		yy = 1.0f;

	      		g2 = norm (dx[i-1][j], dy[i-1][j]);
	      		g4 = norm (dx[i+1][j], dy[i+1][j]);
	      		if (xc*yc > 0.0){
					g3 = norm (dx[i+1][j+1], dy[i+1][j+1]);
					g1 = norm (dx[i-1][j-1], dy[i-1][j-1]);
	      		} else{
					g3 = norm (dx[i+1][j-1], dy[i+1][j-1]);
					g1 = norm (dx[i-1][j+1], dy[i-1][j+1]);
	      		}

	    	} else{

				//die x-Komponente ist größer,
				//also ist der Gradient horizontal ausgerichtet
	      		xx = Math.abs(yc)/Math.abs(xc);
	      		yy = 1.0f;

	      		g2 = norm (dx[i][j+1], dy[i][j+1]);
	      		g4 = norm (dx[i][j-1], dy[i][j-1]);
	      		if (xc*yc > 0.0){
					g3 = norm (dx[i-1][j-1], dy[i-1][j-1]);
					g1 = norm (dx[i+1][j+1], dy[i+1][j+1]);
	      		} else{
					g1 = norm (dx[i-1][j+1], dy[i-1][j+1]);
					g3 = norm (dx[i+1][j-1], dy[i+1][j-1]);
	      		}
	    	}

			//Berechne den interpolierten Wert der Gradientenmagnitude
	    	if ( (g > (xx*g1 + (yy-xx)*g2)) && (g > (xx*g3 + (yy-xx)*g4)) ){
	      		if (g*mag_scale <= 255)
					mag[i][j] = (int)(g*mag_scale);
	      		else
					mag[i][j] = 255;
	      	} else 
				mag[i][j] = 0;
			
  			}
		}
	}
	
	//HYSTERESIS thresholding der Kantenpixel. 
	//Bei einem Pixel mit einem Wert größer als der obere Schwellwert
	//wird angefangen eine verbunden Folge von Pixels zu suchen,
	//die einen Wert größer als den unteren Schwellwert haben.
	private int[][] hysteresis(int[][] mag){
	
		int[][] im = new int[breite][hoehe];
		
		if (high < low){
	  		estimateThresh(mag);
	  	}
	  	
		//für jede Kante(Pixel) mit einer Magnitude > obere Schwellwert
		//suche Kantenpixel die über dem unteren Schwellwert liegen
   		
   		//wenn der obere und der untere Threshold beide null sind
   		//dann ist jeder Pixel Kante, denn jeder Pixel hat 
   		//einen Grauwert >= 0
   		//das Kantenbild ist vollkommen weiß
   		if(/*(high == 0) && */(low == 0)){
   			for(int i=breite-1; i >= 0; i--)
   				Arrays.fill(im[i], 255);
   		}
   		
   		
   		//wenn low==0, dann gibt es nur einen Thresholdwert
   		//damit sind alle Pixel mit einem Grauwert >= high Kanten,
   		//die anderen nicht
   		//gleicher Fall low == high --> nur ein Threshold
   		else if((low==high) && (high != 0)){// || ((low == 0) && (high > 0))){
   			for(int i=breite-1; i >= 0; i--)
   				for(int j=hoehe-1; j >= 0; j--)
   					if(mag[i][j] >= high)
   						im[i][j] = 255;
   			}
   			else{
   				for (int i=breite-1; i >= 0; i--)
	  				for (int j=hoehe-1; j >= 0; j--)
	    				if (mag[i][j] >= high){
	    					trace(i, j, im, mag);
						}
				boolean b = false;
				do{
					b = false;
					for (int i=breite-1; i >= 0; i--)
	  					for (int j=hoehe-1; j >= 0; j--)
							if(im[i][j] == 255)
								if(trace(i, j, im, mag))
									b = true;
				} while (b);
			}
		return im;
	}
	
	//Suche die Kantenpixel, die einen Wert > untere Schwellwert haben
	private boolean trace(int i, int j, int[][] im, int[][] mag){
		
		boolean flag = false;
		
		im[i][j] = 255;
		mag[i][j] = 0;
		for (int n= -1; n <= 1; n++){
	    	for(int m= -1; m <=1 ; m++){
				if (i==0 && m==0) continue;
				if (range(i+n, j+m) && (mag[i+n][j+m] >= low)){
					im[i+n][j+m] = 255;
					mag[i+n][j+m] = 0;
					flag = true;
				}
			}
		}
		
		return flag;
	}
	
	private void estimateThresh(int[][] mag){
		
		//Histogramm vom Magnitudenbild erstellen
		int[] hist = new int[256];
		for(int i=width; i < breite-width; i++)
	 		for(int j=width; j < hoehe-width; j++)
	 			hist[mag[i][j]]++;
		
		int j = (int)(0.9*breite*hoehe)-hist[0];
		int k = 255;
						
		int count = hist[k];
		while (count < j){
			k++;
	  		if (k < 0) break;
	  		count += hist[k];
		}
		
		high = k;

		int i=0;
		while (hist[i]==0) i++;

		low = (high+i)/2;
	}
	
	//Test, ob der Index des Pixels im Bildbereich ist
	private boolean range(int i, int j){
		if ((i<0) || (i >= breite)) return false;
		if ((j<0) || (j >= hoehe)) return false;
		return true;
	}
}