//Fractal.java
//Texturerkennung über den Hurst-Koeffizienten

import java.awt.*;
import java.util.*;

class Fractal extends Algorithmus{
	
	Fractal(Image img, BildKomp ziel, int threshold){
		super(img, ziel, 7, threshold);
	}
	
	protected void algorithmus(int[][] pixs, int[][] hilfe){
		for (int i=3; i < alt_breite-3; i++)
	  		for (int j=3; j < alt_hoehe-3; j++){
	    		int z = ((int)Hurst(pixs, i, j))*64;
	    		max = z > max ? z : max;
	    		min = z < min ? z : min;
				hilfe[i][j] = z;
			}
	}

	// Fractal Maße
	private float Hurst(int[][] im, int c, int r) {
		float[] data = new float[7];
		float[] y = {0.0f, 0.34657359f, 0.69314718f, 0.80471896f,
							1.0397208f, 1.0986123f, 1.1512925f};
		int max, min;

		max = min  = im[c+1][r];
		
		//Distanz 1
		int[] minmax4 = new int[4];
		minmax4[0] = im[c+1][r];
		minmax4[1] = im[c-1][r];
		minmax4[2] = im[c][r-1];
		minmax4[3] = im[c][r+1];
		Arrays.sort(minmax4);
		if(minmax4[3] > max)
			max = minmax4[3];
		if(minmax4[0] < min)
			min = minmax4[0];
		if ((max-min)!= 0) data[0] = (float)Math.log(max-min);
		
		//Distanz sqrt(2)
		minmax4[0] = im[c-1][r-1];
		minmax4[1] = im[c+1][r-1];
		minmax4[2] = im[c-1][r+1];
		minmax4[3] = im[c+1][r+1];
		Arrays.sort(minmax4);
		if(minmax4[3] > max)
			max = minmax4[3];
		if(minmax4[0] < min)
			min = minmax4[0];
		if ((max-min)!= 0) data[1] = (float)Math.log(max-min);
		
		//Distanz 2
		minmax4[0] = im[c][r-2];
		minmax4[1] = im[c][r+2];
		minmax4[2] = im[c-2][r];
		minmax4[3] = im[c+2][r];
		Arrays.sort(minmax4);
		if(minmax4[3] > max)
			max = minmax4[3];
		if(minmax4[0] < min)
			min = minmax4[0];
		if ((max-min)!= 0) data[2] = (float)Math.log(max-min);

		
		//Distanz sqrt(5)
		int[] minmax8 = new int[8];
		minmax8[0] = im[c+2][r-1]; 
		minmax8[1] = im[c-2][r-1];
		minmax8[2] = im[c+2][r+1];
		minmax8[3] = im[c-2][r+1];
		minmax8[4] = im[c+1][r-2];
		minmax8[5] = im[c-1][r-2];
		minmax8[6] = im[c+1][r+2];
		minmax8[7] = im[c-1][r+2];
		Arrays.sort(minmax8);
		if(minmax8[7] > max)
			max = minmax8[7];
		if(minmax8[0] < min)
			min = minmax8[0];		
		if ((max-min)!=0) data[3] = (float)Math.log(max-min);
		
		//Distanz sqrt(8)
		minmax4[0] = im[c-2][r+2];
		minmax4[1] = im[c+2][r+2];
		minmax4[2] = im[c-2][r-2];
		minmax4[3] = im[c+2][r-2];
		Arrays.sort(minmax4);
		if(minmax4[3] > max)
			max = minmax4[3];
		if(minmax4[0] < min)
			min = minmax4[0];
		if ((max-min)!= 0) data[4] = (float)Math.log(max-min);
		
		//Distanz 3
		minmax4[0] = im[c][r+3];
		minmax4[1] = im[c][r-3];
		minmax4[2] = im[c-3][r];
		minmax4[3] = im[c+3][r];
		Arrays.sort(minmax4);
		if(minmax4[3] > max)
			max = minmax4[3];
		if(minmax4[0] < min)
			min = minmax4[0];
		if ((max-min)!= 0) data[5] = (float)Math.log(max-min);
		
		//Distanz sqrt(10)
		minmax8[0] = im[c-1][r+3]; 
		minmax8[1] = im[c+1][r+3];
		minmax8[2] = im[c+1][r-3];
		minmax8[3] = im[c-1][r-3];
		minmax8[4] = im[c+3][r-1];
		minmax8[5] = im[c-3][r-1];
		minmax8[6] = im[c+3][r+1];
		minmax8[7] = im[c-3][r+1];
		Arrays.sort(minmax8);
		if(minmax8[7] > max)
			max = minmax8[7];
		if(minmax8[0] < min)
			min = minmax8[0];			
		if ((max-min)!=0) data[6] = (float)Math.log(max-min);
	
		// bestimmen der LSBF Geraden für log(dG) und log(Distanz)
		return fit (y, data, 7);
	}
	
	//Methode der kleinsten Quadrate
	private float fit (float[] x, float[] y, int ndata){
		float s1=0.0f, s2=0.0f, s3=0.0f, xstrich = 0.0f, ystrich = 0.0f;
	
		for(int i=ndata-1; i >= 0; i--){
	  		xstrich += x[i];
	  		ystrich += y[i];
		}
		xstrich /= ndata;
		ystrich /= ndata;

		for(int i=ndata-1; i >= 0; i--){
	  		s1 = (x[i] - xstrich);
	  		s2 += s1*(y[i]-ystrich);
	  		s3 += s1*s1;
		}
		return s2/s3;	//Steigung
	}
}