//GLCM.java

import java.awt.*;
import java.util.*;

class GLCM extends Algorithmus{
	private int wn;
	private int dir;
	private int distance;
	private int algo;
	private float Mean;
	private float[] Sum = new float[511];	//Summenhistogramm
	private float[] Diff = new float[511];	//Differenzhistogramm
	private Win w = new Win();
	
	GLCM(Image img, BildKomp ziel, int algo, int maskengroesse, int distance, int dir, int threshold){
		super(img, ziel, maskengroesse, threshold);
		this.wn = maskengroesse/2;
		this.dir = dir;
		this.distance = distance;
		this.algo = algo;
	}
	
	protected byte[] berechneAlgo(byte[] img){
		int[][] im = Util.feldZuIntMatrix(img, alt_breite, alt_hoehe);
		float[][] res = new float[alt_breite][alt_hoehe];
		
		float erg = 0;
		for(int i=wn; i <= alt_hoehe-wn-1; i++){
			for(int j=wn; j <= alt_breite-wn-1; j++){
				w.top = i-wn; w.bottom = i+wn;
				w.left = j-wn; w.right = j+wn;
				erg = apply(im);
				maxf = erg > maxf ? erg : maxf;
	    		minf = erg < minf ? erg : minf;
				res[j][i] = erg;
			}
		}
		rescaling(res);
		 
		img = Util.matrixZuFeld(res);
		Threshold.thresholdSchwarzWeiss(img, threshold, Threshold.EDGE_WHITE);
		return img;
	}
	
	private float apply(int[][] im){
		
		//Histogramme für alle 4 Richtungen berechnen
		int k = 0;
		int p1, p2;
		Arrays.fill(Sum, 0);
		Arrays.fill(Diff, 0);
		
		for (int y = w.top; y <= w.bottom; y++) 
	  		for (int x = w.left; x <= w.right; x++){ 
	  			p1 = im[x][y];
	  				
				//horizontal
	    		if ((dir == 0) && (x+distance <= w.right))
	    			p2 = im[x + distance][y];
				//vertikal
	    		else if ((dir == 2) && (y+distance <= w.bottom))
	    			p2 = im[x][y+distance];
				//45° diagonal
	    		else if ((dir==1) && (y+distance <= w.bottom) && (x-distance >= w.left))
	    			p2 = im[x-distance][y+distance];
				//135° diagonal
	    		else if ((dir == 3) && (y+distance <= w.bottom) && (x+distance <= w.right))
		    		p2 = im[x+distance][y+distance];
	    		else continue;
	    		k++;
	    		
	    		Sum[p1+p2]++;
	    		Diff[p1-p2+255]++;
	    		
	   		}
	   		
	   
	   		
		//Normalisieren
		float sum = 0.0f;
		for (int i=510; i >=0; i--){
	  		Sum[i] /= k;
	  		sum += Sum[i]*i;
	  		Diff[i] /= k;
		}
	
		Mean = sum/2f;
		
		switch(algo){
			case 0: return Mean; 
			case 1: return standardAbweichung();
			case 2: return maximaleWahrscheinlichkeit();
			case 3: return energy(); 
			case 4: return contrast(); 
			case 5: return homogeneity(); 
			case 6: return entropy(); 
			default: return 0f;
		}
	}

	//Summen- und Differenzhistogramm
	private float standardAbweichung(){
		float s1, s2, meanmean, var;
	
		s1 = s2 = 0.0f;
		meanmean = Mean+Mean;

		for (int i=510; i>=0; i--){
	  		s1 += (i-meanmean)*(i-meanmean)*Sum[i];
	  		int j = i-255;
	  		s2 += j*j*Diff[i];
		}
		var = (s1 + s2)/2.0f;
		return (float)Math.sqrt(var);
	}

	//Summenhistogramm
	private float maximaleWahrscheinlichkeit(){
		float y = 0f;
		for (int i=510; i>=0; i--)
	    	y = y < Sum[i] ? Sum[i] : y;
		return y;
	}

	//Summen- und Differenzhistogramm
	private float energy(){
		float y=0.0f, z = 0.0f;

		for(int i=510; i>=0; i--){
	  		y += Sum[i]*Sum[i];
	  		z += Diff[i]*Diff[i];
		}
	
		return y*z;
	}

	//Differenzhistogramm
	private float contrast(){
		float y=0.0f;

		for(int i=510, j=i-255; i>=0; i--, j--)
	  		y += j*j*Diff[i];
	  		
		return y;
	}

	//Differenzhistogramm
	private float homogeneity(){
		float y = 0.0f;

		for(int i=510, j=i-255; i>=0; i--, j--)
	  		y += 1.0f/(1.0f + j*j) * Diff[i];
		
		return y;
	}

	//Summen- und Differenzhistogramm
	private float entropy (){
		float  h1=0.0f, h2=0.0f;

		for(int i=510; i>=0; i--){
	  		if(Sum[i] > 0)
	    		h1 += Sum[i]* (float)(Math.log(Sum[i]));
			if (Diff[i] > 0)
	    		h2 += Diff[i]*(float)(Math.log(Diff[i]));
	    }
		return -h1 - h2;
	}
}