//Glaettung.java
//Komponente, die für die Bildglättung
//verantwortlich ist
//verwendete Algorithmen: Median-Filter, Gauß-Filter

import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.awt.*;

class Glaettung extends JPanel{

	private  BildKomp smoothed;
	private  Oberflaeche ob;
	
	private JRadioButton r_gauss, r_nichts;
	private JComboBox c_maske;
	private JLabel l_maske, l_median, l_gauss, l_anzeige;
	private JButton b_apply;
	private ButtonGroup g;
	
	private int mask = 3;
	private static int anzahlMedian = 0;
	private static int anzahlGauss = 0;
	
	Glaettung(final Oberflaeche ob){
		
		this.ob = ob;
		this.smoothed = ob.getSmoothed();
				
		//JRadioButton
		//1: Median
		JRadioButton r_median = new JRadioButton("Median Filter", false);
		r_median.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				activateSmoothing(true);
			}
		});
		
		//2: Gauss
		r_gauss = new JRadioButton("Gaussian Filter", false);;
		r_gauss.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				activateSmoothing(true);
			}
		});
	
		//3: keine Glättung
		r_nichts = new JRadioButton("reset (no smoothing)", true);
		r_nichts.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				reset();
				smoothed.setImage(ob.getOriginal().getImage());
				SwingUtilities.invokeLater(new Runnable(){
					public void run(){
						ob.getAlgorithmen().neuberechnen();
					}
				});
			}
		});
	
		//logische Gruppierung der RadioButtons
		g = new ButtonGroup();
		g.add(r_nichts);
		g.add(r_median);
		g.add(r_gauss);
	
		//JComboBox
		//1: Maskengrösse
		c_maske = new JComboBox();
		c_maske.setMaximumRowCount(2);
		c_maske.addItem("3x3");
		c_maske.addItem("5x5");
		JScrollPane sp = new JScrollPane(c_maske);
		c_maske.setEnabled(false);
		c_maske.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				String s = (String)c_maske.getSelectedItem();
				if(s != null){
					if(s.equalsIgnoreCase("3x3"))
						mask = 3;
					else
						mask = 5;
				}
			}
		});
		
		//JButton
		//ausführen der Algorithmen
		b_apply = new JButton("Apply");
		b_apply.setEnabled(false);
		b_apply.setToolTipText("Apply smoothing");
		b_apply.addActionListener(new ButtonListener());
		
		//JLabel
		//1: für die ComboBox mit den Maskengroessen
		l_maske = new JLabel("Filter mask size: ", JLabel.LEFT);
		l_maske.setEnabled(false);
		
		//2: für die Algorithmen
		JLabel l_algo = new JLabel("Choose a smoothing filter", JLabel.LEFT);
		
		//3: für die Anzeige, wie oft Algos ausgeführt wurden
		l_median = new JLabel("No smoothing applied");
		l_median.setEnabled(false);
		l_gauss = new JLabel(" ");
		l_gauss.setEnabled(false);
		l_anzeige = new JLabel(" ");
		l_anzeige.setEnabled(false);
			
		JPanel p = new JPanel();
		p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
		p.add(smoothed);
		p.add(Box.createVerticalStrut(12));
						
		JPanel sk = new JPanel();
		sk.setLayout(new GridLayout(0,1, 0, 0));
		sk.add(l_algo);
		sk.add(r_nichts);
		sk.add(r_median);
		sk.add(r_gauss);
		p.add(sk);
				
		JPanel l = new JPanel();
		l.setLayout(new FlowLayout(FlowLayout.LEFT));
		sk = new JPanel();
		sk.setLayout(new GridLayout(2, 1, 0, 8));
		sk.add(l_maske);
		sk.add(c_maske);
		sk.add(new JLabel());
		sk.add(b_apply);
		l.add(sk);
		p.add(l);
		
		p.add(Box.createVerticalStrut(12));
				
		JPanel k = new JPanel();
		JPanel t = new JPanel();
		t.setLayout(new GridLayout(0,1));
		t.add(l_median);
		t.add(l_gauss);
		t.add(l_anzeige);
		k.add(t);
		k.setBorder(BorderFactory.createCompoundBorder(new EtchedBorder(), new EmptyBorder(6,6,6,6)));
		p.add(k);
		
		add(p, BorderLayout.NORTH);
	}
	
	private void activateSmoothing(boolean activ){
		c_maske.setEnabled(activ);
		b_apply.setEnabled(activ);
		l_gauss.setEnabled(activ);
		l_median.setEnabled(activ);
		l_maske.setEnabled(activ);
		l_anzeige.setEnabled(activ);
	}
	
	public void reset(){
		activateSmoothing(false);
		r_nichts.setSelected(true);
		anzahlGauss = 0;
		anzahlMedian = 0;
		l_median.setText("No smoothing applied");
		l_gauss.setText(" ");
		l_anzeige.setText(" ");
	}
	
	class ButtonListener implements ActionListener{
		public void actionPerformed(ActionEvent e){
			final Thread t;
			if(g.isSelected(r_gauss.getModel())){
				t = new Thread(new Gauss(smoothed.getImage(), smoothed, mask));
				anzahlGauss++;
			}
			else{
				t = new Thread(new Median(smoothed.getImage(), smoothed, mask));
				anzahlMedian++;
			}
			SwingUtilities.invokeLater(new Runnable(){
				public void run(){
					t.start();
					try{
						t.join();
					}catch (InterruptedException ex){}
					ob.getAlgorithmen().neuberechnen();
				}
			});
			l_median.setText("Median Filter "+anzahlMedian+"x applied");
			l_gauss.setText("Gaussian Filter "+anzahlGauss+"x applied");
			l_anzeige.setText("Filter mask size: "+mask+"x"+mask+" pixel");
		}
	}
}		