//Kirsch.java
//Implementation des Kirsch-Kanten-Detektoren

import java.awt.Image;
import java.util.*;

class Kirsch extends Algorithmus{
		
	Kirsch(Image img, BildKomp ziel, int threshold){
		super(img, ziel, 3, threshold);
	}
	
	protected void algorithmus(int[][] pixs, int[][] hilfe){
		for(int x=breite-2; x > 0; x--)
			for(int y=hoehe-2; y > 0; y--){
				int z = masks(pixs, y, x);
				max = z > max ? z : max;
	    		min = z < min ? z : min;
		   		hilfe[x][y] = z;
		   	}
	}

	//alle Masken auf Pixel(row, col) anwenden
	//und größten Wert zurückgeben
	// (p00 p10 p20)
	// (p01 p11 p21)
	// (p02 p12 p22)
	private int masks(int[][] pixs, int row, int col){
		
		int resp[] = new int[8];
		
		int c00 = col-1;
		int c01 = col+1;
		int r00 = row-1;
		int r01 = row+1;
		
		int p003 = 3*pixs[c00][r00];
		int p005 = 5*pixs[c00][r00];
		int p103 = 3*pixs[col][r00];
		int p105 = 5*pixs[col][r00];
		int p203 = 3*pixs[c01][r00];
		int p205 = 5*pixs[c01][r00];
		
		int p013 = 3*pixs[c00][row];
		int p015 = 5*pixs[c00][row];
		int p213 = 3*pixs[c01][row];
		int p215 = 5*pixs[c01][row];
		
		int p023 = 3*pixs[c00][r01];
		int p025 = 5*pixs[c00][r01];
		int p123 = 3*pixs[col][r01];
		int p125 = 5*pixs[col][r01];
		int p223 = 3*pixs[c01][r01];
		int p225 = 5*pixs[c01][r01];
				
		resp[0] = -p003-p103+p205
				  -p013     +p215
				  -p023-p123+p225;
				  
		resp[1] = -p003+p105+p205
				  -p013     +p215
				  -p023-p123-p223;
		
		resp[2] =  p005+p105+p205
				  -p013     -p213
				  -p023-p123-p223;
		
		resp[3] =  p005+p105-p203
				  +p015     -p213
				  -p023-p123-p223;
		
		resp[4] = p005-p103-p203
				 +p015     -p213
				 +p025-p123-p223;
				  
		resp[5] = -p003-p103-p203
				  +p015     -p213
				  +p025+p125-p223;
				   
		resp[6] = -p003-p103-p203
				  -p013     -p213
				  +p025+p125+p225;
				   
		resp[7] = -p003-p103-p203
				  -p013     +p215
				  -p023+p125+p225;
		
		Arrays.sort(resp);
		
		return resp[7];
	}
}