//Menue.java
//Menü des Applets

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.applet.*;
import java.util.*;
import java.io.*;

class Menue extends JMenuBar{

	private URL codebase;
	private UIManager.LookAndFeelInfo [] info;
	private static Vector registeredFrames = new Vector();
	
	private Segmentation f;
	private Browser b;
	private Menu_ActionListener mal;
	
	Menue(Segmentation f){	
		
		this.f = f;
		this.codebase = f.getCodebase();
		this.mal = new Menu_ActionListener();
					
		//die einzelnen Menüpunkte
		
		JMenu fileMenu = new JMenu("File");
		JMenu lafMenu = new JMenu("Look and Feel");
		JMenu algorithmMenu = new JMenu("Algorithms");
		JMenu helpMenu = new JMenu("?");
		JMenu edgeMenu = new JMenu("Edge Detection");
		JMenu texturMenu = new JMenu("Texture Analysis");
		JMenu smoothMenu = new JMenu("Smoothing Algorithms");
		
		//Optionen des "look and feel"-Menüs
		
		ButtonGroup buttongroup = new ButtonGroup();
		
		//Determining the Available Look and Feels
		info = UIManager.getInstalledLookAndFeels();
		for (int i = 0; i < info.length; i++){
			String humanReadableName = info[i].getName();
			String className = info[i].getClassName();
			JRadioButtonMenuItem r_look = new JRadioButtonMenuItem(humanReadableName);
			r_look.addActionListener(mal);
			lafMenu.add(r_look);
			buttongroup.add(r_look);
			if(i==0)
				r_look.setSelected(true);
		}
		
		// "Open"-Option im "File"-Menü
		JMenuItem menuitem = new JMenuItem("Open...");
		menuitem.addActionListener(mal);
		fileMenu.add(menuitem);
		fileMenu.addSeparator();
		
		
		// "Quit"-Option im "File"-Menü
		menuitem = new JMenuItem("Quit");
		menuitem.addActionListener(mal);
		fileMenu.add(menuitem);

		//Algorithmenmenü
		//Kanten
		menuitem = new JMenuItem("Roberts Edge Detector");
		menuitem.addActionListener(mal);
		edgeMenu.add(menuitem);
		menuitem = new JMenuItem("Derivative Operators");
		menuitem.addActionListener(mal);
		edgeMenu.add(menuitem);
		menuitem = new JMenuItem("Prewitt Edge Detector");
		menuitem.addActionListener(mal);
		edgeMenu.add(menuitem);
		menuitem = new JMenuItem("Sobel Edge Detector");
		menuitem.addActionListener(mal);
		edgeMenu.add(menuitem);
		menuitem = new JMenuItem("Robinson Edge Detector");
		menuitem.addActionListener(mal);
		edgeMenu.add(menuitem);
		menuitem = new JMenuItem("Kirsch Edge Detector");
		menuitem.addActionListener(mal);
		edgeMenu.add(menuitem);
		menuitem = new JMenuItem("Canny Edge Detector");
		menuitem.addActionListener(mal);
		edgeMenu.add(menuitem);
		
		//Glättung
		menuitem = new JMenuItem("Median Filter");
		menuitem.addActionListener(mal);
		smoothMenu.add(menuitem);
		menuitem = new JMenuItem("Gaussian Filter");
		menuitem.addActionListener(mal);
		smoothMenu.add(menuitem);
		
		//Textur
		menuitem = new JMenuItem("Fractal Dimension");
		menuitem.addActionListener(mal);
		texturMenu.add(menuitem);
		menuitem = new JMenuItem("GLCM");
		menuitem.addActionListener(mal);
		texturMenu.add(menuitem);
		menuitem = new JMenuItem("Law's Texture Energy Measures");
		menuitem.addActionListener(mal);
		texturMenu.add(menuitem);
		
		algorithmMenu.add(smoothMenu);
		algorithmMenu.add(edgeMenu);
		algorithmMenu.add(texturMenu);
		
		// Option "About" im Hilfemenü
		menuitem = new JMenuItem("Index");
		menuitem.addActionListener(mal);
		helpMenu.add(menuitem);
		helpMenu.add(algorithmMenu);
		helpMenu.addSeparator();
		menuitem = new JMenuItem("About");
		menuitem.addActionListener(mal);
		helpMenu.add(menuitem);
		
		// die Menüleiste selber
		add(fileMenu);
		add(lafMenu);
		add(helpMenu);
	} 
	
	private void browserStarten(final String seite){
		SwingUtilities.invokeLater(new Runnable(){
			public void run(){
				if(b == null){
					b = new Browser(codebase, seite);
					b.setSize(new Dimension(570, 600));
					b.setTitle("Help");
				}
				else
					b.setPage(codebase, seite);
				b.setVisible(true); 
			}
		});
	}
	
	public static void registerFrame(Window frame){
		registeredFrames.add(frame);
	}
	
	class Menu_ActionListener implements ActionListener{
		private String plaf;
		public void actionPerformed(ActionEvent evt){	
			if (evt.getSource() instanceof JMenuItem){
				String arg = evt.getActionCommand();
		
				//Reaktion auf die Menüeintrage
				if (arg.equals("Quit")){
					f.dispose();
				}
				if (arg.equals("About")){
					String message = "Still Image Segmentation\n\n"+
					  			  "Susanne Krabbe\n"+
					  		       "(c) 2000 University of Mannheim\n ";
					JOptionPane.showMessageDialog(null,message,"About Still Image Segmentation",
					JOptionPane.INFORMATION_MESSAGE); 				
				}
				
				if (arg.equals("Open...")){
					f.getOberflaeche().getBildAuswahl().open();
				}
				
				if (arg.equals("Index"))
					browserStarten("algorithms.html");
					
				if (arg.equals("Derivative Operators"))
					browserStarten("derivative.html");
					
				if (arg.equals("Canny Edge Detector"))
					browserStarten("canny.html");
					
				if (arg.equals("Sobel Edge Detector"))
					browserStarten("sobel.html");
					
				if (arg.equals("Kirsch Edge Detector"))
					browserStarten("kirsch.html");
				
				if (arg.equals("Roberts Edge Detector"))
					browserStarten("robert.html");
				
				if (arg.equals("Robinson Edge Detector"))
					browserStarten("robinson.html");
					
				if (arg.equals("Prewitt Edge Detector"))
					browserStarten("prewitt.html");
				
				if (arg.equals("Gaussian Filter"))
					browserStarten("gauss.html");
					
				if (arg.equals("Median Filter"))
					browserStarten("median.html");
					
				if (arg.equals("Fractal Dimension"))
					browserStarten("fractal.html");
					
				if (arg.equals("GLCM"))
					browserStarten("glcm.html");
					
				if (arg.equals("Law's Texture Energy Measures"))
					browserStarten("laws.html");
				
			}		
			if (evt.getSource() instanceof JRadioButtonMenuItem){			
				String arg = evt.getActionCommand();
				//'look and feel' Menüeinträge
				for (int i=0; i<info.length; i++){
					String name = info[i].getName();
					if(arg.equals(name)){
						plaf = info[i].getClassName();
						break;
					}
				}
				
				try{
					UIManager.setLookAndFeel(plaf);
					SwingUtilities.updateComponentTreeUI(f);
					validate();
					for(int i=0; i < registeredFrames.size(); i++){
						Window frame = (Window)registeredFrames.elementAt(i);
						SwingUtilities.updateComponentTreeUI(frame);
						validate();
					}
						
				} catch(Exception e){	
					String message = "LookAndFeel switch failed!\n\n"+
					                 "Maybe the pluggable LaF component is not available.";
					JOptionPane.showMessageDialog(null , message, "Error",
					JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}
}
